
#include "Study/study_1/study_1RangeStudy.h"

//******************************************************
//Global Variables
//******************************************************
double alpha;
double beta;
double lambda_i;
double lambda_w;

//********************************************************
//study_1RangeStudy Constructor
//********************************************************
study_1RangeStudy::study_1RangeStudy() {

  // define arrays of global variable names and types
  NumGVs = 4;
  NumExps = 10;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("alpha");
  GVTypes[0]=strdup("double");
  GVNames[1]=strdup("beta");
  GVTypes[1]=strdup("double");
  GVNames[2]=strdup("lambda_i");
  GVTypes[2]=strdup("double");
  GVNames[3]=strdup("lambda_w");
  GVTypes[3]=strdup("double");

  // create the arrays to store the values of each gv
  alphaValues = new double[NumExps];
  betaValues = new double[NumExps];
  lambda_iValues = new double[NumExps];
  lambda_wValues = new double[NumExps];

  // call methods to assign values to each gv
  SetValues_alpha();
  SetValues_beta();
  SetValues_lambda_i();
  SetValues_lambda_w();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//study_1RangeStudy Destructor
//******************************************************
study_1RangeStudy::~study_1RangeStudy() {
  delete [] alphaValues;
  delete [] betaValues;
  delete [] lambda_iValues;
  delete [] lambda_wValues;
  delete ThePVModel;
}


//******************************************************
// set values for alpha
//******************************************************
void study_1RangeStudy::SetValues_alpha() {
  alphaValues[0] = 1000.0;
  alphaValues[1] = 1000.0;
  alphaValues[2] = 1000.0;
  alphaValues[3] = 1000.0;
  alphaValues[4] = 1000.0;
  alphaValues[5] = 1000.0;
  alphaValues[6] = 1000.0;
  alphaValues[7] = 1000.0;
  alphaValues[8] = 1000.0;
  alphaValues[9] = 1000.0;
}


//******************************************************
// set values for beta
//******************************************************
void study_1RangeStudy::SetValues_beta() {
  betaValues[0] = 10000.0;
  betaValues[1] = 10000.0;
  betaValues[2] = 10000.0;
  betaValues[3] = 10000.0;
  betaValues[4] = 10000.0;
  betaValues[5] = 10000.0;
  betaValues[6] = 10000.0;
  betaValues[7] = 10000.0;
  betaValues[8] = 10000.0;
  betaValues[9] = 10000.0;
}


//******************************************************
// set values for lambda_i
//******************************************************
void study_1RangeStudy::SetValues_lambda_i() {
  lambda_iValues[0] = 1.0E-7;
  lambda_iValues[1] = 1.0E-7;
  lambda_iValues[2] = 1.0E-7;
  lambda_iValues[3] = 1.0E-7;
  lambda_iValues[4] = 1.0E-7;
  lambda_iValues[5] = 1.0E-7;
  lambda_iValues[6] = 1.0E-7;
  lambda_iValues[7] = 1.0E-7;
  lambda_iValues[8] = 1.0E-7;
  lambda_iValues[9] = 1.0E-7;
}


//******************************************************
// set values for lambda_w
//******************************************************
void study_1RangeStudy::SetValues_lambda_w() {
  lambda_wValues[0] = 1.0E-6;
  lambda_wValues[1] = 2.0E-6;
  lambda_wValues[2] = 3.0E-6;
  lambda_wValues[3] = 4.0E-6;
  lambda_wValues[4] = 4.9999999999999996E-6;
  lambda_wValues[5] = 5.999999999999999E-6;
  lambda_wValues[6] = 6.999999999999999E-6;
  lambda_wValues[7] = 8.0E-6;
  lambda_wValues[8] = 9.0E-6;
  lambda_wValues[9] = 1.0E-5;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void study_1RangeStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "alpha\tdouble\t" << alpha << endl;
  cout << "beta\tdouble\t" << beta << endl;
  cout << "lambda_i\tdouble\t" << lambda_i << endl;
  cout << "lambda_w\tdouble\t" << lambda_w << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *study_1RangeStudy::GetGVValue(char *TheGVName) {
  if (strcmp("alpha", TheGVName) == 0)
    return &alpha;
  else if (strcmp("beta", TheGVName) == 0)
    return &beta;
  else if (strcmp("lambda_i", TheGVName) == 0)
    return &lambda_i;
  else if (strcmp("lambda_w", TheGVName) == 0)
    return &lambda_w;
  else 
    cerr<<"!! study_1RangeStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void study_1RangeStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("alpha", TheGVName) == 0)
    alpha = *(double *)TheGVValue;
  else if (strcmp("beta", TheGVName) == 0)
    beta = *(double *)TheGVValue;
  else if (strcmp("lambda_i", TheGVName) == 0)
    lambda_i = *(double *)TheGVValue;
  else if (strcmp("lambda_w", TheGVName) == 0)
    lambda_w = *(double *)TheGVValue;
  else 
    cerr<<"!! study_1RangeStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void study_1RangeStudy::SetGVs(int expNum) {
  alpha = alphaValues[expNum];
  beta = betaValues[expNum];
  lambda_i = lambda_iValues[expNum];
  lambda_w = lambda_wValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new study_1RangeStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* study_1RangeStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new reliabPVModel(expandTimeArrays);
  return ThePVModel;
}


